@@ -6,6 +6,7 @@ from .strategy import strategy
 from .version import require_version
 from .helpers import listdir
 from .pathgenerator import PathGenerator
+from .pathinterpolation import PathInterpolation, interpolate
 from .simulation import run_sim, plot_sim, SIM_DEFAULT_PARAMS
 
 from .. import (
@@ -15,7 +16,7 @@ from .. import (
     __date__,
 )
 
-def print_version(require=None, all=True):
+def print_version(require=None, all=False):
     """
     prints Carbon version numbers; calls require_version(require) if not require is None
     """
@@ -27,5 +28,16 @@ def print_version(require=None, all=True):
     if not require is None:
         require_version(require)
 
-
+def plt_style(style, alt_style):
+    """
+    calls plt.style.use(style) and as fallback alt_style, all wrapped in try blocks
+    """
+    from matplotlib import pyplot as plt
+    try:
+        plt.style.use(style)
+    except:
+        try:
+            plt.style.use(alt_style)
+        except:
+            print("[plt_style] both styles failed", style, alt_style)
 
